package dnd;

import java.awt.Color;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;

public class ColoredShape {

  protected Shape shape;
  protected Color color;
  protected boolean filled;

  public ColoredShape(Shape shape, Color color) {
    this(shape, color, true);
  }

  public ColoredShape(Shape shape, Color color, boolean filled) {
    this.shape = shape;
    this.color = color;
    this.filled = filled;
  }

  public Shape getShape() {
    return shape;
  }

  public Color getColor() {
    return color;
  }

  public boolean isFilled() {
    return filled;
  }

  public static ColoredShape createRectangle(int x, int y, int w, int h,
      Color color, boolean filled) {
    return new ColoredShape(new Rectangle2D.Double(x, y, w - 1, h - 1), color,
        filled);
  }

  public static ColoredShape createRoundRectangle(int x, int y, int w, int h,
      double arcw, double arch, Color color, boolean filled) {
    return new ColoredShape(new RoundRectangle2D.Double(x, y, w - 1, h - 1,
        arcw, arch), color, filled);
  }

  public static ColoredShape createEllipse(int x, int y, int w, int h,
      Color color, boolean filled) {
    return new ColoredShape(new Ellipse2D.Double(x, y, w - 1, h - 1), color,
        filled);
  }

  public static ColoredShape createCircle(int x, int y, int radius,
      Color color, boolean filled) {
    return createEllipse(x, y, radius, radius, color, filled);
  }
}
