package dnd;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

public class DragGroup {

  private final Set<DragItem> dragItems = new HashSet<DragItem>();

  public DragGroup() {
  }

  public DragGroup(DragItem... items) {
    addAll(Arrays.asList(items));
  }

  public DragGroup(Collection<DragItem> items) {
    addAll(items);
  }

  public void add(DragItem item) {
    if (item != null && dragItems.add(item)) {
      item.addGroup(this);
    }
  }

  public void remove(DragItem item) {
    if (item != null && dragItems.remove(item)) {
      item.removeGroup(this);
    }
  }

  public void addAll(Collection<DragItem> items) {
    for (DragItem item : items) {
      add(item);
    }
  }

  public void removeAll(Collection<DragItem> items) {
    for (DragItem item : items) {
      remove(item);
    }
  }

  public boolean contains(DragItem item) {
    return dragItems.contains(item);
  }

  public void enable() {
    for (DragItem item : dragItems) {
      item.setDragFlags(item.getDragFlags() | DragItem.DRAG_GROUP_ENABLED);
    }
  }

  public void disable() {
    for (DragItem item : dragItems) {
      item.setDragFlags(item.getDragFlags() & ~DragItem.DRAG_GROUP_ENABLED);
    }
  }
}
