package dnd;

import java.awt.Component;
import java.awt.Image;
import java.awt.Point;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import javax.swing.Action;

public abstract class DragItem {

  public final static int DRAG_EXCLUSIVE = 0x1;
  public final static int DRAG_GROUP_ENABLED = 0x2;

  protected Action action;
  protected Image image;
  protected int dragFlags;
  protected Set<DragGroup> dragGroups;
  protected Point glassPaneLocation;

  private final Set<DropListener> listeners = new HashSet<DropListener>();

  public DragItem() {
  }

  public DragItem(Action action) {
    this.action = action;
  }

  public DragItem(Action action, Image image) {
    this(action);
    this.image = image;
  }

  public void addDropListener(DropListener listener) {
    if (listener != null)
      listeners.add(listener);
  }

  public void removeDropListener(DropListener listener) {
    if (listener != null)
      listeners.remove(listener);
  }

  public DropListener[] getDropListeners() {
    return listeners.toArray(new DropListener[listeners.size()]);
  }

  public Action getAction() {
    return action;
  }

  public void setAction(Action action) {
    this.action = action;
  }

  public Image getImage() {
    return image;
  }

  public void setImage(Image image) {
    this.image = image;
  }

  public int getDragFlags() {
    return dragFlags;
  }

  public void setDragFlags(int dragFlags) {
    this.dragFlags = dragFlags;
  }

  public Collection<DragGroup> getGroups() {
    return dragGroups;
  }

  public void addGroup(DragGroup dragGroup) {
    if (dragGroups == null) {
      dragGroups = new HashSet<DragGroup>();
    }
    if (dragGroup != null && dragGroups.add(dragGroup)) {
      dragGroup.add(this);
    }
  }

  public void removeGroup(DragGroup dragGroup) {
    if (dragGroups != null && dragGroup != null && dragGroups.remove(dragGroup)) {
      dragGroup.remove(this);
    }
  }

  public Point getGlassPaneLocation() {
    return glassPaneLocation;
  }

  public void setGlassPaneLocation(Point glassPaneLocation) {
    this.glassPaneLocation = glassPaneLocation;
  }

  public abstract void updateImage(Component c);
}