package dnd;

import static dnd.DragText.JustificationEnum.CENTER;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.image.BufferedImage;

import javax.swing.Action;

public class DragText extends DragItem {

  private static final int arcWidth = 8;
  private static final int arcHeight = 4;
  private static final Color lightYellow = Color.yellow.brighter();

  public enum JustificationEnum {
    LEFT {
      @Override
      public int getOffset(int textLineWidth, int textAreaWidth) {
        return 0;
      }
    },
    CENTER {
      @Override
      public int getOffset(int textLineWidth, int textAreaWidth) {
        return (textAreaWidth - textLineWidth) / 2;
      }
    },
    RIGHT {
      @Override
      public int getOffset(int textLineWidth, int textAreaWidth) {
        return textAreaWidth - textLineWidth;
      }
    };

    abstract int getOffset(int textLineWidth, int textAreaWidth);
  }

  protected String text;
  protected String[] textLines;
  protected Color bg;
  protected JustificationEnum justification;

  public DragText(Action action, String text) {
    this(action, text, lightYellow, CENTER);
  }

  public DragText(Action action, String text, JustificationEnum justification) {
    this(action, text, lightYellow, justification);
  }

  public DragText(Action action, String text, Color bg) {
    this(action, text, bg, CENTER);
  }

  public DragText(Action action, String text, Color bg,
      JustificationEnum justification) {
    super(action);
    setText(text);
    this.bg = bg;
    this.justification = justification;
  }

  public String getText() {
    return text;
  }

  public String[] getTextLines() {
    return textLines.clone();
  }

  public void setText(String text) {
    this.text = text;
    this.textLines = text.split("\n");
    image = null; // image doit tre recalcule
  }

  public Color getBg() {
    return bg;
  }

  public void setBg(Color bg) {
    this.bg = bg;
    image = null; // image doit tre recalcule
  }

  public JustificationEnum getJustification() {
    return justification;
  }

  public void setJustification(JustificationEnum justification) {
    this.justification = justification;
    image = null; // image doit tre recalcule
  }

  @Override
  public void updateImage(Component c) {
    if (image != null || textLines.length == 0)
      return;

    Font font = c.getFont();
    FontMetrics fm = c.getFontMetrics(font);

    Dimension textAreaSize = new Dimension();
    int[] textLineWidths = new int[textLines.length];
    int i = 0;
    for (String line : textLines) {
      Dimension size = font.getStringBounds(line, fm.getFontRenderContext())
          .getBounds().getSize();

      textAreaSize.width = Math.max(textAreaSize.width, size.width);
      textAreaSize.height += size.height;
      textLineWidths[i++] = size.width;
    }

    int width = textAreaSize.width + arcWidth;
    int height = textAreaSize.height + arcHeight;

    image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
    Graphics g = image.getGraphics();

    g.setColor(bg);
    g.fillRoundRect(0, 0, width, height, arcWidth, arcHeight);
    g.setColor(c.getForeground());
    g.drawRoundRect(0, 0, width - 1, height - 1, arcWidth, arcHeight);
    g.setFont(font);

    int textLineHeight = textAreaSize.height / textLines.length;
    int y = textLineHeight - arcHeight / 2;
    int j = 0;
    for (String line : textLines) {
      int x = justification.getOffset(textLineWidths[j++], textAreaSize.width)
          + arcWidth / 2;
      g.drawString(line, x, y);
      y += textLineHeight;
    }
  }
}