package dnd;

import java.awt.Component;

import dnd.DropEvent.DnDActionEnum;

public class DropAdapter implements DropListener {

  protected final Component component;

  protected int supportsFlags;

  protected RolloverImageBuilder rolloverImageBuilder;

  public DropAdapter(Component component) {
    this(component, SUPPORTS_ALL);
  }

  public DropAdapter(Component component, int supportsFlags) {
    this(component, supportsFlags, null);
  }

  public DropAdapter(Component component,
      RolloverImageBuilder rolloverImageBuilder) {
    this(component, SUPPORTS_ALL, rolloverImageBuilder);
  }

  public DropAdapter(Component component, int supportsFlags,
      RolloverImageBuilder rolloverImageBuilder) {
    this.component = component;
    this.supportsFlags = supportsFlags;
    this.rolloverImageBuilder = rolloverImageBuilder;
  }

  @Override
  public int getSupportsFlags() {
    return supportsFlags;
  }

  public void setSupportsFlags(int supportsFlags) {
    this.supportsFlags = supportsFlags;
  }

  @Override
  public Component getDropTarget() {
    return component;
  }

  @Override
  public RolloverImageBuilder getRolloverImageBuilder() {
    return rolloverImageBuilder;
  }

  public void setRolloverImageBuilder(RolloverImageBuilder rolloverImageBuilder) {
    this.rolloverImageBuilder = rolloverImageBuilder;
  }

  @Override
  public void dropPerformed(DropEvent evt) {
  }

  @Override
  public void dropPerformed(DropCollapsedEvent evt) {
  }

  protected void doDragAction(DropEvent evt) {
    doAnyAction(evt, DnDActionEnum.DRAG);
  }

  protected void doDropAction(DropEvent evt) {
    doAnyAction(evt, DnDActionEnum.DROP);
  }

  protected void doDragnDropAction(DropEvent evt) {
    doAnyAction(evt, DnDActionEnum.DRAGNDROP);
  }

  protected void doLocationAction(DropEvent evt) {
    doAnyAction(evt, DnDActionEnum.LOCATION);
  }

  protected void doAnyAction(DropEvent evt, DnDActionEnum dndAction) {
    if (evt.getAction() != null) {
      evt.getAction().actionPerformed(evt.createActionEvent(dndAction));
    }
  }
}