package dnd;

import java.awt.Component;
import java.awt.Point;
import java.awt.event.ActionEvent;

import javax.swing.Action;

public class DropEvent implements DragDropContainer {

  public enum DnDActionEnum {
    DRAG("DnDActionEnum.drag"), DROP("DnDActionEnum.drop"), DRAGNDROP(
        "DnDActionEnum.dragndrop"), LOCATION("DnDActionEnum.location");

    private String actionCommand;

    private DnDActionEnum(String actionCommand) {
      this.actionCommand = actionCommand;
    }

    public String getActionCommand() {
      return actionCommand;
    }
  }

  private final Component dragSource;
  private final Action action;
  private final Component dropTarget;
  private final Point dropLocation;

  public DropEvent(Component dragSource, Action action, Component dropTarget,
      Point dropLocation) {
    this.dragSource = dragSource;
    this.action = action;
    this.dropTarget = dropTarget;
    this.dropLocation = dropLocation;
  }

  public Component getDragSource() {
    return dragSource;
  }

  public Action getAction() {
    return action;
  }

  public Component getDropTarget() {
    return dropTarget;
  }

  public Point getDropLocation() {
    return dropLocation;
  }

  public ActionEvent createActionEvent(DnDActionEnum dndAction) {

    Object source = null;

    switch (dndAction) {
    case DRAG:
      source = dragSource;
      break;

    case DROP:
      source = dropTarget;
      break;

    case DRAGNDROP:
      source = (DragDropContainer) this;
      break;

    case LOCATION:
      source = dropLocation;
      break;
    }

    return new ActionEvent(source, ActionEvent.ACTION_PERFORMED,
        dndAction.actionCommand);
  }
}
