package dnd;

import java.awt.image.RGBImageFilter;

public class NegativeFilter extends RGBImageFilter {

  protected double brightness;

  public NegativeFilter(int brightness) {
    if (brightness < 0 || brightness > 100) {
      throw new IllegalArgumentException(
          "'brightness' doit tre compris entre 0 et 100 : " + brightness);
    }

    // The filter's operation does not depend on the pixel's location,
    // so IndexColorModels can be filtered directly.
    canFilterIndexColorModel = true;
    this.brightness = (100.0 - brightness) / 100.0;
  }

  public int filterRGB(int x, int y, int rgb) {
    int alpha = rgb & 0xff000000;
    int red = (rgb >> 16) & 0xff;
    int green = (rgb >> 8) & 0xff;
    int blue = (rgb >> 0) & 0xff;
    return alpha | brighter(red) << 16 | brighter(green) << 8 | brighter(blue);
  }

  private int brighter(int color) {
    return 0xff - (int) ((0xff - color ^ 0xff) * brightness);
  }
}