package dnd.demo;

import static dnd.DropEvent.DnDActionEnum.DRAGNDROP;
import static dnd.RolloverImageBuilder.ellipse1;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.util.Properties;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

import dnd.DnDGlassPane;
import dnd.DnDManager;
import dnd.DragComponent;
import dnd.DragDropContainer;
import dnd.DragGroup;
import dnd.DragItem;
import dnd.DragPicture;
import dnd.DragText;
import dnd.DropAdapter;
import dnd.DropCollapsedEvent;
import dnd.DropEvent;
import dnd.DropListener;
import dnd.RolloverImageBuilder;
import dnd.UIHelper;
import dnd.DragText.JustificationEnum;
import dnd.RolloverImageBuilder.BorderedImageBuilder;

public class DnDDemo extends JFrame {

  private static final long serialVersionUID = 1L;

  private JLabel corbeillePleine;
  private JLabel corbeilleVide;
  private JButton boutonViderCorbeille;

  public DnDDemo() {
    super("Dmo Drag and Drop");

    Properties properties = new Properties();
    try {
      properties.load(new BufferedInputStream(getClass().getResourceAsStream(
          "demo.properties")));
    } catch (IOException e) {
    }

    DnDGlassPane glassPane = new DnDGlassPane(new BorderedImageBuilder(
        BorderFactory.createLoweredBevelBorder(), Color.pink), properties);
    setGlassPane(glassPane);

    Container contentPane = getContentPane();
    contentPane.setLayout(new BorderLayout());

    DnDManager manager = new DnDManager(glassPane);

    JPanel center = new JPanel();
    contentPane.add(BorderLayout.CENTER, center);
    center.setBorder(BorderFactory.createEtchedBorder());

    corbeillePleine = UIHelper.createLabel(getClass(),
        "images/corbeille052.png");
    corbeillePleine.setVisible(false);
    corbeillePleine.setName("Corbeille");
    center.add(corbeillePleine);

    corbeilleVide = UIHelper.createLabel(getClass(), "images/corbeille051.png");
    corbeilleVide.setVisible(true);
    corbeilleVide.setName("Corbeille vide");
    center.add(corbeilleVide);

    DropAdapter dropCorbeillePleineListener = new CorbeilleDropAdapter(
        corbeillePleine);
    DropAdapter dropCorbeilleVideListener = new CorbeilleDropAdapter(
        corbeilleVide);

    Box boxNorth = Box.createHorizontalBox();
    contentPane.add(BorderLayout.NORTH, boxNorth);
    boxNorth.setBorder(BorderFactory.createTitledBorder("Multi - groupe A"));

    Box boxSouth = Box.createHorizontalBox();
    contentPane.add(BorderLayout.SOUTH, boxSouth);
    boxSouth.setBorder(BorderFactory.createTitledBorder("Mono - groupe B"));

    Draggable[] draggables = {
        new DraggableImage(manager, "Photo (opaque)", "images/hardware001.png"),
        new DraggableImage(manager, "Camra", "images/hardware027.png"),
        new DraggableImage(manager, "CD Rom", "images/hardware087.png"),
        new DraggableImage(manager, "Ecran 1", "images/hardware175.png"),
        new DraggableImage(manager, "Ecran 2", "images/hardware189.png"),
        new DraggableImage(manager, "Ecran 3", "images/hardware232.png"),
        new DraggableComponent(manager, "Tlphone", "images/hardware196.png"),
        new DraggableComponent(manager, "Souris", "images/hardware247.png"),
        new DraggableComponent(manager, "Clavier", "images/hardware255.png"),
        new DraggableComponent(manager, "Joystick", "images/hardware292.png"),
        new DraggableComponent(manager, "Imprimante", "images/hardware375.png"),
        new DraggableComponent(manager, "Ordinateur", "images/hardware385.png"), };

    final DragGroup dgNorth = new DragGroup();
    final DragGroup dgSouth = new DragGroup();

    int i = 0;
    for (Draggable draggable : draggables) {
      if (i < draggables.length / 2) {
        boxNorth.add(draggable.label);
        dgNorth.add(draggable.dragItem);
      } else {
        boxSouth.add(draggable.label);
        dgSouth.add(draggable.dragItem);
      }
      i++;

      draggable.dragItem.addDropListener(dropCorbeillePleineListener);
      draggable.dragItem.addDropListener(dropCorbeilleVideListener);

      for (Draggable draggable2 : draggables) {
        if (draggable2 != draggable) {
          draggable2.dragItem.addDropListener(draggable.adapter);
        }
      }
    }

    JPanel aide = new JPanel();
    contentPane.add(BorderLayout.WEST, aide);
    aide.setBorder(BorderFactory.createTitledBorder("Aide"));

    JLabel label = UIHelper.createLabel(getClass(), "images/software150.png");
    label.setName("Aide");
    final DragText dragText = new DragText(new DisplayAction("Aide"),
        "Dmo Drag and Drop !\n"
            + "\nMulti - lignes\n1234567890+&'(-_)=~#{[|`\\^@]}\n"
            + "azertyuiop^$\n" + "qsdfghjklm%*\n" + "wxcvbn,?;.:/!\n"
            + "AZERTYUIOPQSDFGHJKLMWXCVBN\n\n" + "\n"
            + "\n" + "\n"
            + "\n" + "");
    manager.register(label, dragText);
    dragText.addDropListener(dropCorbeillePleineListener);
    dragText.addDropListener(dropCorbeilleVideListener);
    aide.add(label);

    Box buttons = Box.createVerticalBox();
    contentPane.add(BorderLayout.EAST, buttons);
    buttons.setBorder(BorderFactory.createTitledBorder("Corbeille"));

    boutonViderCorbeille = new JButton(
        new CorbeilleAction("Vider la corbeille"));
    boutonViderCorbeille.setEnabled(false);
    boutonViderCorbeille.setName("Vider");
    buttons.add(Box.createVerticalGlue());
    buttons.add(boutonViderCorbeille);
    buttons.add(Box.createVerticalGlue());

    JButton button = new JButton(
        "<html><center>Dposez moi<br>dans la corbeille<br>(drag exclusif)</html>");
    button.setName("Dposez");
    DragComponent dragComponent = new DragComponent(
        new DisplayAction("Dposez"));
    dragComponent.setDragFlags(DragItem.DRAG_EXCLUSIVE);
    manager.register(button, dragComponent);
    dragComponent.addDropListener(dropCorbeillePleineListener);
    dragComponent.addDropListener(dropCorbeilleVideListener);
    buttons.add(button);
    buttons.add(Box.createVerticalGlue());

    JMenuBar mb = new JMenuBar();
    setJMenuBar(mb);
    JMenu menu = new JMenu("File");
    mb.add(menu);
    JMenuItem item = new JMenuItem(new AbstractAction("Exit") {
      private static final long serialVersionUID = 1L;

      @Override
      public void actionPerformed(ActionEvent e) {
        System.exit(0);
      }
    });
    menu.add(item);
    menu = new JMenu("Rollover");
    mb.add(menu);
    for (Draggable draggable : draggables) {
      JMenu subMenu = new JMenu(draggable.text);
      menu.add(subMenu);
      draggable.populateMenu(subMenu);
    }
    menu = new JMenu("Groupes");
    mb.add(menu);
    menu.add(createCheckBoxGroup("A actif ?", dgNorth));
    menu.add(createCheckBoxGroup("B actif ?", dgSouth));

    menu = new JMenu((String) dragText.getAction().getValue(Action.NAME));
    mb.add(menu);
    JMenu subMenu = new JMenu("Alignement");
    menu.add(subMenu);
    for (final JustificationEnum je : JustificationEnum.values()) {
      item = new JMenuItem(new AbstractAction(je.name()) {
        private static final long serialVersionUID = 1L;

        @Override
        public void actionPerformed(ActionEvent e) {
          dragText.setJustification(je);
        }
      });
      subMenu.add(item);
    }
    item = new JMenuItem(new AbstractAction("Texte") {
      private static final long serialVersionUID = 1L;

      @Override
      public void actionPerformed(ActionEvent e) {
        final JDialog dialog = new JDialog(DnDDemo.this,
            "Entrez le texte de l'aide", true);
        final JTextArea ta = new JTextArea(dragText.getText());
        dialog.getContentPane().add(
            new JScrollPane(ta, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
                ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED));

        JButton b = new JButton(new AbstractAction("Valider") {
          private static final long serialVersionUID = 1L;

          @Override
          public void actionPerformed(ActionEvent e) {
            dragText.setText(ta.getText());
            dialog.dispose();
          }
        });
        dialog.getContentPane().add(b, BorderLayout.SOUTH);

        dialog.setMinimumSize(new Dimension(320, 200));
        dialog.pack();
        dialog.setLocationRelativeTo(DnDDemo.this);
        dialog.setVisible(true);
      }
    });
    menu.add(item);

    pack();
    setResizable(false);
    setLocationRelativeTo(null);
    setDefaultCloseOperation(EXIT_ON_CLOSE);
  }

  private JCheckBoxMenuItem createCheckBoxGroup(String text,
      final DragGroup dragGroup) {
    JCheckBoxMenuItem cb = new JCheckBoxMenuItem(text);
    cb.addItemListener(new ItemListener() {

      @Override
      public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED) {
          dragGroup.enable();
        } else {
          dragGroup.disable();
        }
      }
    });
    return cb;
  }

  class CorbeilleAction extends AbstractAction {

    private static final long serialVersionUID = 1L;

    CorbeilleAction(String name) {
      super(name);
    }

    @Override
    public void actionPerformed(ActionEvent e) {
      corbeilleVide.setVisible(true);
      corbeillePleine.setVisible(false);
      boutonViderCorbeille.setEnabled(false);
    }
  }

  class CorbeilleDropAdapter extends DropAdapter {

    CorbeilleDropAdapter(Component component) {
      super(component, RolloverImageBuilder.negative15);
    }

    @Override
    public void dropPerformed(DropCollapsedEvent evt) {
      StringBuilder sb = new StringBuilder();
      for (Component c : evt.getDragSources()) {
        sb.append(c.getName()).append(", ");
      }
      sb.replace(sb.length() - 2, sb.length(), " dans ").append(
          evt.getDropTarget().getName());
      JOptionPane.showMessageDialog(evt.getDropTarget(), sb);

      corbeilleVide.setVisible(false);
      corbeillePleine.setVisible(true);
      boutonViderCorbeille.setEnabled(true);
    }
  }

  static class Draggable {

    String text;
    JLabel label;
    DragItem dragItem;
    DropAdapter adapter;

    Draggable(String text, String icon, int supportsFlags) {
      this.text = text;
      label = UIHelper.createLabel(getClass(), text, icon);
      label.setName(text);

      adapter = new DropAdapter(label, supportsFlags) {
        @Override
        public void dropPerformed(DropEvent evt) {
          doDragnDropAction(evt);
        }
      };
    }

    public void populateMenu(JMenu menu) {
      JMenuItem item = new JMenuItem(new MenuAction("rec0", adapter,
          RolloverImageBuilder.rec0));
      menu.add(item);
      item = new JMenuItem(new MenuAction("rec1", adapter,
          RolloverImageBuilder.rec1));
      menu.add(item);
      menu.add(new JSeparator());
      item = new JMenuItem(new MenuAction("roundRec0", adapter,
          RolloverImageBuilder.roundRec0));
      menu.add(item);
      item = new JMenuItem(new MenuAction("roundRec1", adapter,
          RolloverImageBuilder.roundRec1));
      menu.add(item);
      menu.add(new JSeparator());
      item = new JMenuItem(new MenuAction("ellipse0", adapter,
          RolloverImageBuilder.ellipse0));
      menu.add(item);
      item = new JMenuItem(new MenuAction("ellipse1", adapter, ellipse1));
      menu.add(item);
      menu.add(new JSeparator());
      item = new JMenuItem(new MenuAction("negative15", adapter,
          RolloverImageBuilder.negative15));
      menu.add(item);
      menu.add(new JSeparator());
      item = new JMenuItem(new MenuAction("raised etched", adapter,
          RolloverImageBuilder.raisedEtched));
      menu.add(item);
      item = new JMenuItem(new MenuAction("lowered etched", adapter,
          RolloverImageBuilder.loweredEtched));
      menu.add(item);
      item = new JMenuItem(new MenuAction("raised bevel", adapter,
          RolloverImageBuilder.raisedBevel));
      menu.add(item);
      item = new JMenuItem(new MenuAction("lowered bevel", adapter,
          RolloverImageBuilder.loweredBevel));
      menu.add(item);
    }
  }

  static class DraggableImage extends Draggable {

    DraggableImage(DnDManager manager, String text, String icon) {
      super(text, icon, DropListener.SUPPORTS_MULTI_SOURCE);

      dragItem = new DragPicture(new DisplayAction(text), getClass(), icon);
      manager.register(label, dragItem);
      label.setForeground(Color.BLUE);
    }
  }

  static class DraggableComponent extends Draggable {

    DraggableComponent(DnDManager manager, String text, String icon) {
      super(text, icon, DropListener.SUPPORTS_NONE);

      dragItem = new DragComponent(new DisplayAction(text));
      manager.register(label, dragItem);
      label.setForeground(Color.MAGENTA);
    }
  }

  static class DisplayAction extends AbstractAction {

    private static final long serialVersionUID = 1L;

    public DisplayAction(String name) {
      super(name);
    }

    @Override
    public void actionPerformed(ActionEvent e) {
      if (DRAGNDROP.getActionCommand().equals(e.getActionCommand())) {
        DragDropContainer container = (DragDropContainer) e.getSource();
        JOptionPane.showMessageDialog(container.getDropTarget(), container
            .getDragSource().getName()
            + " vers " + container.getDropTarget().getName());
      }
    }
  }

  static class MenuAction extends AbstractAction {

    private static final long serialVersionUID = 1L;

    DropAdapter adapter;
    RolloverImageBuilder builder;

    public MenuAction(String name, DropAdapter adapter,
        RolloverImageBuilder builder) {
      super(name);
      this.adapter = adapter;
      this.builder = builder;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
      adapter.setRolloverImageBuilder(builder);
    }
  }

  public static void main(String[] args) {
    DnDDemo demo = new DnDDemo();
    demo.setVisible(true);
  }
}